@@ -3,6 +3,13 @@ import pickle
 import sys
 import os
 import opschema
+import random
+import numpy as np
+
+def list_schemas():
+    print('Available Schemas in ops directory')
+    for op_path in opschema.list_schemas():
+        print(op_path)
 
 def gen_input(op_path, out_dir):
     op = opschema.init_op(op_path)
@@ -30,32 +37,37 @@ def test_op(op_path, out_dir, test_id):
                 sys.excepthook(*sys.exc_info())
             break
 
-def validate(op_path, out_dir, test_ids=None):
+def validate(op_path, out_dir, test_ids=None, skip_ids=None, dtype_err_quota=2):
     opschema.register(op_path)
     op = opschema.get(op_path)
 
-    if test_ids is None:
-        test_ids = set()
-    elif isinstance(test_ids, int):
+    if isinstance(test_ids, int):
         test_ids = {test_ids}
-    else:
+    elif isinstance(test_ids, tuple):
         test_ids = set(test_ids)
-    op.validate(out_dir, test_ids)
+
+    if isinstance(skip_ids, int):
+        skip_ids = {skip_ids}
+    elif isinstance(skip_ids, tuple):
+        skip_ids = set(skip_ids)
+
+    return op.validate(out_dir, test_ids, skip_ids, dtype_err_quota)
 
 def explain(op_path, include_inventory=False):
     return opschema.explain(op_path, include_inventory)
 
 def main():
-    cmd = sys.argv.pop(1)
-    if cmd == 'gen_input':
-        fire.Fire(gen_input)
-    elif cmd == 'test_op':
-        fire.Fire(test_op)
-    elif cmd == 'validate':
-        fire.Fire(validate)
-    elif cmd == 'explain':
-        fire.Fire(explain)
+    func_map = { 
+            'list': list_schemas,
+            'explain': explain,
+            'gen_input': gen_input,
+            'test_op': test_op,
+            'validate': validate
+            }
+    fire.Fire(func_map)
 
 if __name__ == '__main__':
+    random.seed(192384938948348)
+    np.random.seed(982348)
     main()
 
