@@ -68,7 +68,7 @@ class Index(object):
         return range(min_dim, max_dim + 1)
 
     def display_name(self, full=False):
-        return base.snake_case(self.desc) if full else  self.idx
+        return base.snake_case(self.desc) if full else self.idx
 
 class Partial(object):
     """
@@ -90,7 +90,7 @@ class OpSchema(object):
         self.avail_edits = 0
         self.avail_test_edits = 0
         self.max_yield_count = 1000
-        self.comp_dims_mode = True
+        self.comp_dims_mode = None 
 
         # error quotas
         self.dtype_err_quota = 2
@@ -428,66 +428,66 @@ class OpSchema(object):
         report = '\n'.join(table)
         return report 
 
+    def _gen_dims_nodes(self):
+        nodes = []
+        for sig, node in self.dims_graph.items():
+            if isinstance(node.func, ge.GenDims):
+                nodes.append(node)
+        return nodes
+
+    def _comp_dims_nodes(self):
+        nodes = []
+        for node in fgraph._topo_sort(self.dims_graph.values()):
+            if isinstance(node.func, ge.CompDims):
+                nodes.append(node)
+        return nodes
+
+    def _dims_input_nodes(self):
+        nodes = []
+        for node in self.dims_graph.values():
+            if isinstance(node.func, ge.DimsInput):
+                nodes.append(node)
+        return nodes
+
     def comp_dims_report(self, use_full_names=False):
         """
         Show a listing of all computed index dimensions 
         """
-        all_nodes = self.dims_graph.values()
-        inp_nodes = [n for n in all_nodes if isinstance(n.func, ge.DimsInput)]
-        inp_nodes = [n for n in inp_nodes if n.sub_name != base.INDEX_RANKS]
-        comp_nodes = [n for n in all_nodes if isinstance(n.func, ge.CompDims)]
-
+        comp_nodes = self._comp_dims_nodes() 
         if len(comp_nodes) == 0:
-            return ''
+            return 'None'
 
+        render = base.RenderCompDims(self)
+        inp_nodes = self._dims_input_nodes()
+        inp_nodes = [n for n in inp_nodes if n.sub_name != base.INDEX_RANKS]
         inp_names = [n.sub_name for n in inp_nodes]
+        self.dims_graph_input[base.INDEX_RANKS] = {}
+
+        # The generative nodes outside the comp_dims graph
         gen_nodes = [n.func.gen_node for n in inp_nodes if n.func.gen_node is
                 not None]
 
-        for sig, node in self.dims_graph.items():
-            if not isinstance(node.func, ge.GenDims):
-                continue
-            info = tuple(self.index[idx].display_name(use_full_names) for idx
-                    in sig)
-            if len(info) == 1:
-                info = info[0]
-            node.set_cached(info)
-
-        vals_list = fgraph.all_values(*gen_nodes)
-        self.comp_dims_mode = False
         comp_formulas = {}
+        vals_list = fgraph.all_values(*gen_nodes)
         for vals in vals_list:
             for name, val in zip(inp_names, vals):
                 if isinstance(val, OpArg):
                     val = val.value()
                 self.dims_graph_input[name] = val
-
-            for comp_node in comp_nodes:
-                # set values of all parents
-                for pa in comp_node.parents:
-                    if isinstance(pa.func, ge.CompDims):
-                        pidx = pa.func.idx
-                        info = self.index[pidx].display_name(use_full_names)
-                        pa.set_cached(info)
-                live_nodes = inp_nodes + [comp_node]
-                fgen = fgraph.gen_graph_values(live_nodes, [comp_node])
-                formulas = list(fgen)
-                if len(formulas) != 1:
-                    raise SchemaError(
-                        f'Error: should get one result from comp graph')
-                formula = formulas[0][0]
-                idx = comp_node.used_name()
+            if use_full_names:
+                eq_map = render.get_snake()
+            else:
+                eq_map = render.get_olc()
+            for idx, (lhs, rhs) in eq_map.items():
                 cset = comp_formulas.setdefault(idx, set())
-                cset.add(formula)
-        self.comp_dims_mode = True
+                eqn = f'{lhs} = {rhs}'
+                cset.add(eqn)
 
-        formula_groups = []
+        formulas = []
         for idx, cset in comp_formulas.items():
-            lhs = self.index[idx].display_name(use_full_names)
-            group = '\n'.join(f'{lhs} = {c}' for c in cset)
-            formula_groups.append(group)
+            formulas.extend(cset)
 
-        report = '\n'.join(formula_groups)
+        report = '\n'.join(formulas)
         return report
 
     def dtype_rules_report(self):
@@ -573,7 +573,8 @@ class OpSchema(object):
         index_inv = self.index_inventory()
         signature = self.signature_report()
         index_ranks = self.index_ranks_report()
-        computed_dims = self.comp_dims_report()
+        computed_dims_snake = self.comp_dims_report(True)
+        computed_dims_olc = self.comp_dims_report(False)
         dtype_rules = self.dtype_rules_report()
         combo_rules = self.excluded_dtypes_report()
 
@@ -584,8 +585,8 @@ class OpSchema(object):
         finals.append(f'Signatures\n\n{signature}')
         finals.append(f'Index ranks\n\n{index_ranks}')
 
-        if computed_dims is not None:
-            finals.append(f'Computed dimensions\n\n{computed_dims}')
+        if computed_dims_snake is not None:
+            finals.append(f'Computed dimensions\n\n{computed_dims_snake}\n\n{computed_dims_olc}')
 
         finals.append(f'DType Rules\n\n{dtype_rules}')
 
@@ -811,12 +812,14 @@ class OpSchema(object):
             # print(op_args)
             yield op_args[0] # extract tuple element
 
-    def validate(self, out_dir, test_ids):
+    def validate(self, out_dir, test_ids, skip_ids, dtype_err_quota):
         if not os.path.exists(out_dir):
             raise RuntimeError(
                 f'{type(self).__qualname__}: Could not open output path '
                 f'\'{out_dir}\' for report generation')
 
+        self._set_gen_error_quotas(dtype_err_quota)
+
         report_fh = open(os.path.join(out_dir, f'{self.op_path}.txt'), 'w')
         summary_fh = open(os.path.join(out_dir, f'{self.op_path}.sum.txt'), 'w')
         cats = [ 'TP', 'TN', 'FP', 'FN' ]
@@ -825,14 +828,17 @@ class OpSchema(object):
         op_args_gen = self.generate_args()
         # print(f'Generated {len(op_args_list)} test cases')
 
-        for test_num, op_args in enumerate(op_args_gen, 1):
+        for test_id, op_args in enumerate(op_args_gen, 1):
+            if skip_ids is not None and test_id in skip_ids:
+                continue
+
             if test_ids is not None:
                 if len(test_ids) == 0:
                     break
-                if test_num not in test_ids:
+                if test_id not in test_ids:
                     continue
                 else:
-                    test_ids.remove(test_num)
+                    test_ids.remove(test_id)
 
             # print(op_args)
             # self.show_graph_calls = True
@@ -841,7 +847,7 @@ class OpSchema(object):
             try:
                 with stderr_redirector(string_err):
                     self.wrapped_op(**arg_dict)
-            except SchemaError as ex:
+            except (OpSchemaInternalError, SchemaError) as ex:
                 print(string_err.getvalue().decode('UTF-8'))
                 raise ex
             except BaseException as ex:
@@ -859,9 +865,9 @@ class OpSchema(object):
 
             stats[cat] += 1
             progress = '  '.join(f'{c}: {stats[c]:-5d}' for c in cats)
-            print(f'\rTest: {test_num:-5d}  {progress}', end='')
+            print(f'\rTest: {test_id:-5d}  {progress}', end='')
             arg_fields = '\t'.join(f'{k}:{v}' for k,v in op_args.items())
-            call = f'{test_num}\t{cat}\t{arg_fields}'
+            call = f'{test_id}\t{cat}\t{arg_fields}'
             print('\n\n', call, file=report_fh)
             print(f'Framework Msg\n{self.framework_exc_msg}\n', file=report_fh)
             print(self._report(), file=report_fh)
